/*
 * The SmartWeb Framework
 * Copyright (C) 2004-2006
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * For further informations on the SmartWeb Framework please visit
 *
 *                        http://smartweb.sourceforge.net
 */
 
package net.smartlab.web.bean;

import java.io.IOException;
import java.io.StreamTokenizer;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

/**
 * TODO documentation
 * 
 * @author rlogiacco
 */
public abstract class ArrayConverter extends Converter {

	/**
	 * TODO documentation
	 */
	protected final static String model[] = new String[0];

	/**
	 * TODO documentation
	 */
	public ArrayConverter() {
		super();
	}

	/**
	 * <p>
	 * Parse an incoming String of the form similar to an array initializer in
	 * the Java language into a <code>List</code> with individual Strings for each
	 * element, according to the following rules.
	 * </p>
	 * <ul>
	 * <li>The string must have matching '{' and '}' delimiters around a
	 * comma-delimited list of values.</li>
	 * <li>Whitespace before and after each element is stripped.</li>
	 * <li>If an element is itself delimited by matching single or double
	 * quotes, the usual rules for interpreting a quoted String apply.</li>
	 * </ul>
	 * 
	 * @param value String value to be parsed
	 * @return a list of String elements
	 * @throws ConversionException if the syntax of <code>svalue</code> is not
	 *             syntactically valid
	 * @throws NullPointerException if <code>svalue</code> is
	 *             <code>null</code>
	 */
	protected List parseElements(String value) throws ConversionException {
		// Validate the passed argument
		if (value == null) {
			throw new NullPointerException();
		}

		// Trim any matching '{' and '}' delimiters
		value = value.trim();
		if (value.startsWith("{") && value.endsWith("}")) {
			value = value.substring(1, value.length() - 1);
		}
		try {
			// Set up a StreamTokenizer on the characters in this String
			StreamTokenizer tokenizer = new StreamTokenizer(new StringReader(value));
			tokenizer.whitespaceChars(',', ','); // Commas are delimiters
			tokenizer.ordinaryChars('0', '9'); // Needed to turn off numeric flag
			tokenizer.ordinaryChars('.', '.');
			tokenizer.ordinaryChars('-', '-');
			tokenizer.wordChars('0', '9'); // Needed to make part of tokens
			tokenizer.wordChars('.', '.');
			tokenizer.wordChars('-', '-');

			// Split comma-delimited tokens into a List
			ArrayList list = new ArrayList();
			while (true) {
				int type = tokenizer.nextToken();
				if ((type == StreamTokenizer.TT_WORD) || (type > 0)) {
					list.add(tokenizer.sval);
				} else if (type == StreamTokenizer.TT_EOF) {
					break;
				} else {
					throw new ConversionException("Encountered token of type " + type);
				}
			}
			// Return the completed list
			return list;

		} catch (IOException ioe) {
			throw new ConversionException(ioe);
		}
	}

	/**
	 * @see net.smartlab.web.bean.Converter#convert(java.lang.Class,
	 *      java.lang.Object, java.util.Locale)
	 */
	public abstract Object convert(Class type, Object value, Locale locale) throws ConversionException;
}